/**
 * Clase principal del Applet Java
 * @author David Gmez Jimnez
 * @version 2010-12-26
 */

import java.io.*;
import java.util.*;
import java.awt.*;
import java.net.*;
import javax.swing.JApplet;
import java.security.AccessController;

public class Tracert extends JApplet{
	
	private static final long serialVersionUID = 69;
	public static String newline = System.getProperty("line.separator");
	
	
	/**
	 * Contenedores de los resultados
	 */
	private String txtTraza = " ";
	private String txtRuta = " ";
	private String txtDesti = " ";
	private String direccio = " ";
	private String[] txtTrazas;
	
	
	/**
	 * Fonts de lletres tipus "fixed width"
	 */
	Font courier = new Font("Courier New", Font.PLAIN, 14);
	Font lucida = new Font("Lucida Console", Font.PLAIN, 14);
	
	public Tracert() {
		super();
	}
	
	
	/**
	 * Mtodo que lanza la traduccin de una direccin a su IP con los privilegios adecuados.
	 * @param nom Direccin en formato texto que queremos traducir a IP
	 * @return IP de la direccin
	 */
	public String lanzaTraduce(String nom){
		this.txtDesti = (String) AccessController.doPrivileged(new Translation(nom));
		System.out.print("IP destino: "+this.txtDesti+newline);
		return this.txtDesti;
	}
	
	
	/**
	 * Mtodo que lanza la obtencin de la ruta a a una IP con los privilegios adecuados.
	 * @param direccion Destino final
	 * @param SO Sistema Operativo del usuario
	 * @return IPs que componen la ruta hasta el destino
	 */
	public String lanzaRuta(String direccion, String SO){
		this.txtRuta =(String) AccessController.doPrivileged(new Route(direccion, SO));
		System.out.print(newline+"IP destino: "+this.txtRuta+newline);		
		txtTrazas = txtRuta.split(",");
		this.paint(this.getGraphics());
		return this.txtRuta;
	}
	
	
	/**
	 * Mtodo que lanza la traza a una direccin con los privilegios adecuados.
	 * @param direccion Direccin a la que queremos hacer la traza
	 * @param SO Sistema Operativo del usuario
	 * @return Salida de la ejecucin del comando
	 */
	public String lanzaTraza(String direccion, String SO){
		this.txtTraza = (String) AccessController.doPrivileged(new Trace(direccion, SO));
		txtTrazas = txtTraza.split(newline);
		this.paint(this.getGraphics());
		return txtTraza;
	}
	
	
	/**
	 * Mtodo que lanza un ping a una direccin con los privilegios adecuados.
	 * @param direccion Direccin a la que queremos hacer el ping
	 * @param SO Sistema Operativo del usuario
	 * @return Salida de la ejecucin del comando
	 */
	public String lanzaPing(String direccion, String SO){
		this.txtTraza = (String) AccessController.doPrivileged(new Ping(direccion, SO));
		txtTrazas = txtTraza.split(newline);
		this.paint(this.getGraphics());
		return txtTraza;
	}
	
	
	/**
	 * Mtodo que lanza un escaneo de puertos a una direccin con los privilegios adecuados.
	 * @param direccion Direccin a la que queremos hacer el escaneo
	 * @return Salida de la ejecucin del comando
	 */
	public String lanzaNmap(String direccion){
		this.txtTraza = (String) AccessController.doPrivileged(new Nmap(direccion));
		txtTrazas = txtTraza.split(newline);
		this.paint(this.getGraphics());
		return txtTraza;
	}
	
	
	/**
	 * Mtodo que obtiene la ruta a una direccin IP.
	 * Se van haciendo pings con caducidad TTL para ver que IP responde en cada paso
	 * @param direccion Destino final
	 * @return IPs que componen la ruta hasta el destino
	 */
	public String ruta(String direccion){  
		Integer ttl = 0;		
		String respDestino = new String();
		String coincidencia = new String();				
		ArrayList<String> lista = new ArrayList<String>();
		String ipDesti = this.traduce(direccion);
		String respFinal = "Respuesta desde " + ipDesti;
		
		while (!coincidencia.startsWith(respFinal))
		{
			ttl = ttl + 1;
	        try 
	        {  
	            Process p = Runtime.getRuntime().exec("cmd /C ping -n 1 -i " + ttl.toString() + " " + ipDesti.toString()); //Ejecutamos el ping  
	            BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));  
	            String line = null;  
	            while ((line = in.readLine()) != null) 
	            {  
	                if (line.contains("Respuesta desde"))	//Extraemos la IP que nos ha respondido en este paso
	                {
	                	respDestino = line.toString();
	                	coincidencia = line.toString();
	                	int pos = respDestino.indexOf(":");
	                	respDestino = respDestino.substring(0,pos);
	                	pos = respDestino.indexOf(" ");
	                	respDestino = respDestino.substring(pos+1,respDestino.length());
	                	pos = respDestino.indexOf(" ");
	                	respDestino = respDestino.substring(pos+1,respDestino.length());
	                	lista.add(respDestino);
	                }
	            }
	        }
	        catch (IOException e) 
	        {  
	            e.printStackTrace();  
	        }
		}
		System.out.print(lista.toString());
		this.txtRuta = lista.toString();
		return lista.toString();
	}
	
	
	/**
	 * Mtodo que traza la ruta a una direccin IP.
	 * @param direccion Direccin a la que queremos hacer la traza
	 * @return Salida de la ejecucin del comando
	 */
	public String traza(String direccion){
		String respuesta = new String();
		try {  
            Process p = Runtime.getRuntime().exec("cmd /C tracert " + direccion.toString());  //Ejecutamos el comando
            BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));  
            String line = null;  
            while ((line = in.readLine()) != null) {  
                respuesta = respuesta+line.toString()+newline;	//Aadimos la salida del comando al buffer
            }
        } catch (IOException e) {  
            e.printStackTrace();  
        }
        System.out.print(respuesta.toString());
        this.txtTraza = respuesta.toString();
        return respuesta;
	}
	
	
	/**
	 * Mtodo que traduce una direccin a su IP
	 * @param nom Direccin en formato texto que queremos traducir a IP
	 * @return IP de la direccin
	 */
	public String traduce(String nom){
		String ip = new String();
		try { 
			InetAddress inetAddr = InetAddress.getByName(nom); //Traduce la direccin en IP
			ip = inetAddr.getHostAddress().toString();
		} catch (UnknownHostException ex) { 
			ip = "fail";
		}	
		this.txtDesti = ip.toString();
		return ip;
	}
	
	
	/**
	 * Mtodo que se ejecuta al iniciar el Applet
	 */
	public void init(){
		this.setBackground(Color.black);
    	this.getParent().setBackground(Color.black);
		System.out.print("init"+newline);
    }
	
	
	/**
	 * Mtodo que se ejecuta despus de init() al iniciar el Applet
	 */
    public void start(){
    	System.out.print("start"+newline);
    }
	
    
    /**
	 * Mtodo que presenta en pantalla los resultados
	 */
    public void paint(Graphics g) {
    	g.setFont(lucida);
    	g.setColor(Color.white);
    	g.clearRect(WIDTH, HEIGHT, getWidth(), getWidth());
    	System.out.print("dibujando..."+newline);
    	g.drawString("Sortida de lnia de comandes:",10,20);
    	if (txtTrazas.length > 0)
    	{
	    	for (int i=1; i<txtTrazas.length; i++)
	    	{
	    		g.drawString(this.txtTrazas[i],10,(i*14)+20);
	    	}
    	}
    }
}
